@@ -26,9 +26,12 @@
 ##############################################################################
 import os
 import hashlib
-import ConfigParser
+from six.moves import configparser
+import tempfile
 
 from slapos.recipe.librecipe import GenericBaseRecipe
+from slapos.util import str2bytes
+from .certificate_authority import popenCommunicate
 
 class Recipe(GenericBaseRecipe):
 
@@ -38,13 +41,10 @@ class Recipe(GenericBaseRecipe):
     self.ca_private = self.options['ca-private']
     self.ca_certs = self.options['ca-certs']
     self.ca_newcerts = self.options['ca-newcerts']
-    self.ca_crl = self.options['ca-crl']
     self.ca_key_ext = '.key'
     self.ca_crt_ext = '.crt'
 
   def install(self):
-    path_list = []
-
     ca_country_code = self.options.get('country-code', 'XX')
     ca_email = self.options.get('email', 'xx@example.com')
     # XXX-BBB: State by mistake has been configured as string "('State',)"
@@ -75,24 +75,25 @@ class Recipe(GenericBaseRecipe):
     self.createFile(openssl_configuration, self.substituteTemplate(
       self.getTemplateFilename('openssl.cnf.ca.in'), config))
 
-    ca_wrapper = self.createPythonScript(
+    return self.createPythonScript(
       self.options['wrapper'],
-      '%s.certificate_authority.runCertificateAuthority' % __name__,
-      dict(
-        openssl_configuration=openssl_configuration,
-        openssl_binary=self.options['openssl-binary'],
-        certificate=os.path.join(self.ca_dir, 'cacert.pem'),
-        key=os.path.join(self.ca_private, 'cakey.pem'),
-        crl=self.ca_crl,
-        request_dir=self.request_directory
-      )
+      __name__ + '.certificate_authority.runCertificateAuthority',
+      (os.path.join(self.ca_private, 'cakey.pem'),
+       os.path.join(self.ca_dir, 'cacert.pem'),
+       self.options['openssl-binary'],
+       openssl_configuration,
+       self.request_directory)
     )
-    path_list.append(ca_wrapper)
-
-    return path_list
 
 class Request(Recipe):
 
+  def setPath(self):
+    self.request_directory = self.options['requests-directory']
+    self.ca_private = self.options['ca-private']
+    self.ca_certs = self.options['ca-certs']
+    self.ca_key_ext = '.key'
+    self.ca_crt_ext = '.crt'
+
   def _options(self, options):
     if 'name' not in options:
       options['name'] = self.name
@@ -108,17 +109,18 @@ class Request(Recipe):
     request_needed = True
 
     name = self.options['name']
-    hash_ = hashlib.sha512(name).hexdigest()
+    hash_ = hashlib.sha512(str2bytes(name)).hexdigest()
     key = os.path.join(self.ca_private, hash_ + self.ca_key_ext)
     certificate = os.path.join(self.ca_certs, hash_ + self.ca_crt_ext)
 
     # XXX Ugly hack to quickly provide custom certificate/key to everyone using the recipe
     if key_content and cert_content:
+      self._checkCertificateKeyConsistency(key_content, cert_content)
       open(key, 'w').write(key_content)
       open(certificate, 'w').write(cert_content)
       request_needed = False
     else:
-      parser = ConfigParser.RawConfigParser()
+      parser = configparser.RawConfigParser()
       parser.add_section('certificate')
       parser.set('certificate', 'name', name)
       parser.set('certificate', 'key_file', key)
@@ -136,12 +138,24 @@ class Request(Recipe):
 
     path_list = [key_file, cert_file]
     if request_needed:
-      wrapper = self.createPythonScript(
+      wrapper = self.createWrapper(
         self.options['wrapper'],
-        'slapos.recipe.librecipe.execute.execute_wait',
-        [ [self.options['executable']],
-          [certificate, key] ],
+        (self.options['executable'],),
+        wait_list=(certificate, key),
       )
       path_list.append(wrapper)
 
     return path_list
+
+  def _checkCertificateKeyConsistency(self, key, certificate):
+    openssl_binary = self.options.get('openssl-binary', 'openssl')
+
+    # Simple test if the user/certificates are readable and don't raise
+    popenCommunicate((openssl_binary, 'x509', '-noout', '-text'), certificate)
+    popenCommunicate((openssl_binary, 'rsa', '-noout', '-text'), key)
+
+    # Check if the key and certificate match
+    modulus_cert = popenCommunicate((openssl_binary, 'x509', '-noout', '-modulus'), certificate)
+    modulus_key = popenCommunicate((openssl_binary, 'rsa', '-noout', '-modulus'), key)
+    if modulus_cert != modulus_key:
+      raise ValueError("The key and certificate provided don't patch each other. Please check your parameters")
