@@ -32,7 +32,6 @@ import os
 import sys
 import inspect
 import re
-import shutil
 import stat
 from six.moves.urllib.parse import quote
 import itertools
@@ -255,31 +254,3 @@ class GenericBaseRecipe(object):
     url = urlunparse((scheme, netloc, path, params, query, fragment))
 
     return url
-
-  def setLocationOption(self):
-    if not self.options.get('location'):
-      self.options['location'] = os.path.join(
-          self.buildout['buildout']['parts-directory'], self.name)
-
-  def download(self, destination=None):
-    """ A simple wrapper around h.r.download, downloading to self.location"""
-    self.setLocationOption()
-
-    import hexagonit.recipe.download
-    if not destination:
-      destination = self.location
-    if os.path.exists(destination):
-        # leftovers from a previous failed attempt, removing it.
-        self.logger.warning('Removing already existing directory %s',
-                            destination)
-        shutil.rmtree(destination)
-    os.mkdir(destination)
-
-    try:
-      options = self.options.copy()
-      options['destination'] = destination
-      hexagonit.recipe.download.Recipe(
-          self.buildout, self.name, options).install()
-    except:
-      shutil.rmtree(destination)
-      raise
