@@ -32,19 +32,23 @@ import re
 
 from slapos.recipe.librecipe import GenericBaseRecipe
 from slapos.recipe.librecipe.inotify import subfiles
+from slapos.util import str2bytes
 
-# This authority only works with dropbear sshkey generator
-def sshkeys_authority(args):
-  requests_directory = args['requests']
-  keygen_binary = args['sshkeygen']
+# This authority only works with dropbear or openssh sshkey generators
+def sshkeys_authority(request_directory, keygen_binary):
+  if 'openssh' in keygen_binary:
+    authority_type = 'openssh'
+  else:
+    # Keep dropbear for compatibility
+    authority_type = 'dropbear'
 
-  for request_filename in subfiles(requests_directory):
+  for request_filename in subfiles(request_directory):
 
     with open(request_filename) as request_file:
       request = json.load(request_file)
 
     key_type = request.get('type', 'rsa')
-    size = str(request.get('size', 2048))
+    size = str(request.get('size', 4096))
     try:
       private_key = request['private_key']
       public_key = request['public_key']
@@ -54,8 +58,12 @@ def sshkeys_authority(args):
     if not os.path.exists(private_key):
       if os.path.exists(public_key):
         os.unlink(public_key)
-      keygen_cmd = [keygen_binary, '-t', key_type, '-f', private_key,
-                    '-s', size]
+      if authority_type == 'openssh':
+        keygen_cmd = [keygen_binary, '-N', "", '-C', "", '-t', key_type,
+                      '-f', private_key, '-b', size]
+      else:
+        keygen_cmd = [keygen_binary, '-t', key_type, '-f', private_key,
+                      '-s', size]
       # If the keygeneration return an non-zero status, it means there's a
       # big problem. Let's exit in this case
       subprocess.check_call(keygen_cmd, env=os.environ.copy())
@@ -88,18 +96,13 @@ def sshkeys_authority(args):
         public_key_file.write(public_key_value)
 
 
-
 class Recipe(GenericBaseRecipe):
 
   def install(self):
-    args = dict(
-      requests=self.options['request-directory'],
-      sshkeygen=self.options['keygen-binary'],
-    )
-
-    wrapper = self.createPythonScript(self.options['wrapper'],
-      __name__ + '.sshkeys_authority', args)
-    return [wrapper]
+    return self.createPythonScript(self.options['wrapper'],
+      __name__ + '.sshkeys_authority',
+      (self.options['request-directory'],
+       self.options['keygen-binary']))
 
 class Request(GenericBaseRecipe):
 
@@ -110,7 +113,7 @@ class Request(GenericBaseRecipe):
     keys_directory = options['keys-directory']
 
     self.private_key = os.path.join(keys_directory,
-      hashlib.sha256(options['name']).hexdigest())
+      hashlib.sha256(str2bytes(options['name'])).hexdigest())
     self.public_key = self.private_key + '.pub'
 
     options['public-key-value'] = ''
@@ -150,11 +153,9 @@ class Request(GenericBaseRecipe):
     os.symlink(self.private_key, private_key_link)
     # end-XXX
 
-    wrapper = self.createPythonScript(
+    wrapper = self.createWrapper(
       self.options['wrapper'],
-      'slapos.recipe.librecipe.execute.execute_wait',
-      [ [self.options['executable']],
-        [self.private_key, self.public_key] ])
-
+      (self.options['executable'],),
+      wait_list=(self.private_key, self.public_key))
 
     return [request_file, wrapper, public_key_link, private_key_link]
