@@ -25,14 +25,16 @@
 #
 ##############################################################################
 
-import unittest
-import os
+import collections
 import glob
 import json
-import collections
-import slapos.test
-import jsonschema
+import os
+import unittest
 
+import jsonschema
+import six
+
+import slapos.test
 
 def createInstanceParameterSchemaValidatorTest(path):
   # Test that json is a valid json schema, supports several
@@ -59,7 +61,16 @@ def createSoftwareCfgValidatorTest(path, software_cfg_schema):
   # which is defined in schema.json in this directory
   def run(self, *args, **kwargs):
     with open(path, "r") as json_file:
-      jsonschema.validate(json.load(json_file), software_cfg_schema)
+      schema = json.load(json_file)
+      jsonschema.validate(schema, software_cfg_schema)
+
+      # also make sure request and response schemas can be resolved
+      schema.setdefault('$id', 'file://' + path)
+      resolver = jsonschema.RefResolver.from_schema(schema)
+      for software_type_definition in six.itervalues(schema['software-type']):
+        resolver.resolve(software_type_definition['request'])
+        resolver.resolve(software_type_definition['response'])
+
   return run
 
 
