@@ -2,18 +2,18 @@
 """
 import os
 import sys
+import six
 
 
-def makeRecipe(recipe_class, options, name='test', slap_connection=None):
-  """Instanciate a recipe of `recipe_class` with `options` with a buildout
-  mapping containing a python and an empty `slapos-connection` mapping, unless
-  provided as `slap_connection`.
+def makeRecipe(recipe_class, options, name='test', buildout=None):
+  """Instantiate a recipe of `recipe_class` with `options` with a `buildout`
+  mapping containing by default a python and an empty slap-connection.
 
   This function expects the test suite to have set SLAPOS_TEST_EGGS_DIRECTORY
   and SLAPOS_TEST_DEVELOP_EGGS_DIRECTORY environment variables, so that the
   test recipe does not need to install eggs again when using working set.
   """
-  buildout = {
+  _buildout = {
     'buildout': {
       'bin-directory': '',
       'find-links': '',
@@ -32,15 +32,17 @@ def makeRecipe(recipe_class, options, name='test', slap_connection=None):
        'software-release-url': '',
      }
   }
-  if slap_connection is not None:
-    buildout['slap-connection'] = slap_connection
 
-  buildout['buildout']['eggs-directory'] = os.environ['SLAPOS_TEST_EGGS_DIRECTORY']
-  buildout['buildout']['develop-eggs-directory'] = os.environ['SLAPOS_TEST_DEVELOP_EGGS_DIRECTORY']
+  _buildout['buildout']['eggs-directory'] = os.environ['SLAPOS_TEST_EGGS_DIRECTORY']
+  _buildout['buildout']['develop-eggs-directory'] = os.environ['SLAPOS_TEST_DEVELOP_EGGS_DIRECTORY']
+
+  if buildout:
+    for section, _options in six.iteritems(buildout):
+      _buildout.setdefault(section, {}).update(**_options)
 
   # Prevent test from accidentally writing to the buildout's eggs
-  buildout['buildout']['newest'] = False
-  buildout['buildout']['offline'] = True
+  _buildout['buildout']['newest'] = False
+  _buildout['buildout']['offline'] = True
 
-  return recipe_class(buildout=buildout, name=name, options=options)
+  return recipe_class(buildout=_buildout, name=name, options=options)
 
